/*
 * Licensed to the Apache Software Foundation (ASF) under one or more contributor license
 * agreements. See the NOTICE file distributed with this work for additional information regarding
 * copyright ownership. The ASF licenses this file to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance with the License. You may obtain a
 * copy of the License at http://www.apache.org/licenses/LICENSE-2.0 Unless required by applicable
 * law or agreed to in writing, software distributed under the License is distributed on an "AS IS"
 * BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the License
 * for the specific language governing permissions and limitations under the License.
 */
package javax.portlet.faces.component;

import java.io.Serializable;

import javax.faces.context.FacesContext;
import javax.faces.component.NamingContainer;
import javax.faces.component.UIViewRoot;
import javax.faces.context.ExternalContext;

import javax.portlet.faces.Bridge;
import javax.portlet.faces.BridgeUtil;
import javax.portlet.faces.annotation.PortletNamingContainer;


/**
 * <code>UIViewRoot</code> that implements portlet specific <code>NamingContainer</code>
 * that ensures the consumer's unique portlet Id is encoded in all tree components.
 * The class is annotated by <code>javax.portlet.faces.annotation.PortletNamingContainer</code>
 * allowing the bridge to recognize that this specific <code>UIViewRoot</code>
 * implements the behavior.
 */
@PortletNamingContainer
public class PortletNamingContainerUIViewRoot extends UIViewRoot implements Serializable, NamingContainer
{

  //TODO: This should be regenerated each time this is modified.  Can this be added to maven?
  private static final long   serialVersionUID = -4524288011655837711L;
  private static final String PORTLET_ENCODED_NAMESPACE_ID = Bridge.BRIDGE_PACKAGE_PREFIX + "PortletEncodedNamespaceId";
  

  public PortletNamingContainerUIViewRoot()
  {
    super();
    
    getAttributes().put(PORTLET_ENCODED_NAMESPACE_ID, Boolean.FALSE);
  }

  /**
   * NamingContainer semantics worked generically (serviced by subclasses) as long as the class
   * is marked as implementing NamingContainer and we use the portletNamespace Id as
   * (part of) the component's id.
   */

  @Override
  public String getContainerClientId(FacesContext context)
  {
    if (BridgeUtil.isPortletRequest() && ((Boolean) this.getAttributes().get(PORTLET_ENCODED_NAMESPACE_ID)).equals(Boolean.TRUE))
    {
      return super.getContainerClientId(context);
    }
    else
    {
      return null;
    }

  }
  
  @Override
  public void setId(String id)
  {
    if (BridgeUtil.isPortletRequest()) 
    {
      // Turns out some Faces impls (the RI) , on restoreView, manually sets the id from 
      // the extracted state prior to telling the component to restore itself from this state.
      // (At which point the self restore overwrites any id set prior.).  As this manual
      // set is using the already encoded (saved) value we could end up with a doubly
      // encoded id until the restore overwrites.  To avoid this -- first test if
      // its already encoded and don't re-encode.
      Boolean encoded = Boolean.FALSE;
      ExternalContext ec = FacesContext.getCurrentInstance().getExternalContext();
      String namespace = ec.encodeNamespace("");
      if (!id.startsWith(namespace))
      {
        id = namespace + "_" + id;
      }
        // now indicate that this id is encoded
      encoded = Boolean.TRUE;
      getAttributes().put(PORTLET_ENCODED_NAMESPACE_ID, encoded);
    }
    super.setId(id);
  }
  
}
